@@ -141,6 +141,7 @@ const BoxPointer = new Lang.Class({
 
         this._muteInput();
 
+        Tweener.removeTweens(this);
         Tweener.addTween(this, { opacity: fade ? 0 : 255,
                                  xOffset: xOffset,
                                  yOffset: yOffset,
@@ -219,25 +220,8 @@ const BoxPointer = new Lang.Class({
         this.bin.allocate(childBox, flags);
 
         if (this._sourceActor && this._sourceActor.mapped) {
-            this._reposition(this._sourceActor, this._arrowAlignment);
-
-            if (this._shouldFlip()) {
-                switch (this._arrowSide) {
-                case St.Side.TOP:
-                    this._arrowSide = St.Side.BOTTOM;
-                    break;
-                case St.Side.BOTTOM:
-                    this._arrowSide = St.Side.TOP;
-                    break;
-                case St.Side.LEFT:
-                    this._arrowSide = St.Side.RIGHT;
-                    break;
-                case St.Side.RIGHT:
-                    this._arrowSide = St.Side.LEFT;
-                    break;
-                }
-                this._reposition(this._sourceActor, this._arrowAlignment);
-            }
+            this._reposition();
+            this._updateFlip();
         }
     },
 
@@ -404,11 +388,11 @@ const BoxPointer = new Lang.Class({
             cr.setLineWidth(borderWidth);
             cr.stroke();
         }
+
+        cr.$dispose();
     },
 
     setPosition: function(sourceActor, alignment) {
-        this._arrowSide = this._userArrowSide;
-
         // We need to show it now to force an allocation,
         // so that we can query the correct size.
         this.actor.show();
@@ -416,7 +400,8 @@ const BoxPointer = new Lang.Class({
         this._sourceActor = sourceActor;
         this._arrowAlignment = alignment;
 
-        this._reposition(sourceActor, alignment);
+        this._reposition();
+        this._updateFlip();
     },
 
     setSourceAlignment: function(alignment) {
@@ -428,7 +413,10 @@ const BoxPointer = new Lang.Class({
         this.setPosition(this._sourceActor, this._arrowAlignment);
     },
 
-    _reposition: function(sourceActor, alignment) {
+    _reposition: function() {
+        let sourceActor = this._sourceActor;
+        let alignment = this._arrowAlignment;
+
         // Position correctly relative to the sourceActor
         let sourceNode = sourceActor.get_theme_node();
         let sourceContentBox = sourceNode.get_content_box(sourceActor.get_allocation_box());
@@ -552,7 +540,7 @@ const BoxPointer = new Lang.Class({
     _shiftActor : function() {
         // Since the position of the BoxPointer depends on the allocated size
         // of the BoxPointer and the position of the source actor, trying
-        // to position the BoxPoiner via the x/y properties will result in
+        // to position the BoxPointer via the x/y properties will result in
         // allocation loops and warnings. Instead we do the positioning via
         // the anchor point, which is independent of allocation, and leave
         // x == y == 0.
@@ -560,37 +548,47 @@ const BoxPointer = new Lang.Class({
                                     -(this._yPosition + this._yOffset));
     },
 
-    _shouldFlip: function() {
+    _calculateArrowSide: function(arrowSide) {
         let sourceAllocation = Shell.util_get_transformed_allocation(this._sourceActor);
-        let boxAllocation = Shell.util_get_transformed_allocation(this.actor);
-        let boxWidth = boxAllocation.x2 - boxAllocation.x1;
-        let boxHeight = boxAllocation.y2 - boxAllocation.y1;
+        let [minWidth, minHeight, boxWidth, boxHeight] = this._container.get_preferred_size();
         let monitor = Main.layoutManager.findMonitorForActor(this.actor);
 
-        switch (this._arrowSide) {
+        switch (arrowSide) {
         case St.Side.TOP:
-            if (boxAllocation.y2 > monitor.y + monitor.height &&
+            if (sourceAllocation.y2 + boxHeight > monitor.y + monitor.height &&
                 boxHeight < sourceAllocation.y1 - monitor.y)
-                return true;
+                return St.Side.BOTTOM;
             break;
         case St.Side.BOTTOM:
-            if (boxAllocation.y1 < monitor.y &&
+            if (sourceAllocation.y1 - boxHeight < monitor.y &&
                 boxHeight < monitor.y + monitor.height - sourceAllocation.y2)
-                return true;
+                return St.Side.TOP;
             break;
         case St.Side.LEFT:
-            if (boxAllocation.x2 > monitor.x + monitor.width &&
+            if (sourceAllocation.y2 + boxWidth > monitor.x + monitor.width &&
                 boxWidth < sourceAllocation.x1 - monitor.x)
-                return true;
+                return St.Side.RIGHT;
             break;
         case St.Side.RIGHT:
-            if (boxAllocation.x1 < monitor.x &&
+            if (sourceAllocation.y1 - boxWidth < monitor.x &&
                 boxWidth < monitor.x + monitor.width - sourceAllocation.x2)
-                return true;
+                return St.Side.LEFT;
             break;
         }
 
-        return false;
+        return arrowSide;
+    },
+
+    _updateFlip: function() {
+        let arrowSide = this._calculateArrowSide(this._userArrowSide);
+        if (this._arrowSide != arrowSide) {
+            this._arrowSide = arrowSide;
+            this._reposition();
+            Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
+                this._container.queue_relayout();
+                return false;
+            }));
+        }
     },
 
     set xOffset(offset) {
