@@ -1,7 +1,6 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
-const Gio = imports.gi.Gio;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -30,9 +29,6 @@ const CLOSE_BUTTON_FADE_TIME = 0.1;
 
 const DRAGGING_WINDOW_OPACITY = 100;
 
-const BUTTON_LAYOUT_SCHEMA = 'org.gnome.shell.overrides';
-const BUTTON_LAYOUT_KEY = 'button-layout';
-
 // When calculating a layout, we calculate the scale of windows and the percent
 // of the available area the new layout uses. If the values for the new layout,
 // when weighted with the values as below, are worse than the previous layout's,
@@ -260,23 +256,37 @@ const WindowClone = new Lang.Class({
 
     _onScroll : function (actor, event) {
         let direction = event.get_scroll_direction();
-        if (direction == Clutter.ScrollDirection.UP) {
+        let delta;
+
+        if (event.is_pointer_emulated())
+            return;
+
+        if (direction == Clutter.ScrollDirection.DOWN) {
+            delta = -SCROLL_SCALE_AMOUNT;
+        } else if (direction == Clutter.ScrollDirection.UP) {
+            delta = +SCROLL_SCALE_AMOUNT;
+        } else if (direction == Clutter.ScrollDirection.SMOOTH) {
+            let [dx, dy] = event.get_scroll_delta();
+            delta = -dy * 10;
+        }
+
+        if (delta > 0) {
             if (this._zoomStep == undefined)
                 this._zoomStart();
             if (this._zoomStep < 100) {
-                this._zoomStep += SCROLL_SCALE_AMOUNT;
+                this._zoomStep += delta;
+                this._zoomStep = Math.min(100, this._zoomStep);
                 this._zoomUpdate();
             }
-        } else if (direction == Clutter.ScrollDirection.DOWN) {
+        } else if (delta < 0) {
             if (this._zoomStep > 0) {
-                this._zoomStep -= SCROLL_SCALE_AMOUNT;
+                this._zoomStep += delta;
                 this._zoomStep = Math.max(0, this._zoomStep);
                 this._zoomUpdate();
             }
             if (this._zoomStep <= 0.0)
                 this._zoomEnd();
         }
-
     },
 
     _zoomUpdate : function () {
@@ -441,8 +451,6 @@ const WindowOverlay = new Lang.Class({
         this._parentActor = parentActor;
         this._hidden = false;
 
-        this._settings = new Gio.Settings({ schema: BUTTON_LAYOUT_SCHEMA });
-
         this.borderSize = 0;
         this.border = new St.Bin({ style_class: 'window-clone-border' });
 
@@ -535,15 +543,8 @@ const WindowOverlay = new Lang.Class({
         let button = this.closeButton;
         let title = this.title;
 
-        let layout = this._settings.get_string(BUTTON_LAYOUT_KEY);
-        let rtl = Clutter.get_default_text_direction() == Clutter.TextDirection.RTL;
-
-        let split = layout.split(":");
-        let side;
-        if (split[0].indexOf("close") > -1)
-            side = rtl ? St.Side.RIGHT : St.Side.LEFT;
-        else
-            side = rtl ? St.Side.LEFT : St.Side.RIGHT;
+        let layout = Meta.prefs_get_button_layout();
+        let side = layout.left_buttons.indexOf(Meta.ButtonFunction.CLOSE) > -1 ? St.Side.LEFT : St.Side.RIGHT;
 
         let buttonX;
         let buttonY = cloneY - (button.height - button._overlap);
