@@ -63,7 +63,8 @@ typedef enum {
   SHELL_CURSOR_DND_UNSUPPORTED_TARGET,
   SHELL_CURSOR_DND_MOVE,
   SHELL_CURSOR_DND_COPY,
-  SHELL_CURSOR_POINTING_HAND
+  SHELL_CURSOR_POINTING_HAND,
+  SHELL_CURSOR_CROSSHAIR
 } ShellCursor;
 
 void    shell_global_set_cursor              (ShellGlobal         *global,
@@ -136,8 +137,6 @@ void     shell_global_reexec_self               (ShellGlobal  *global);
 
 const char *     shell_global_get_session_mode  (ShellGlobal  *global);
 
-ClutterActor * shell_global_create_xrootpmap_texture (ShellGlobal *global);
-
 G_END_DECLS
 
 #endif /* __SHELL_GLOBAL_H__ */
