@@ -37,61 +37,7 @@ typedef struct _StScrollViewFadeClass  StScrollViewFadeClass;
 
 #define DEFAULT_FADE_OFFSET 68.0f
 
-static const gchar *fade_glsl_shader =
-"uniform sampler2D tex;\n"
-"uniform float height;\n"
-"uniform float width;\n"
-"uniform float offset_bottom;\n"
-"uniform float offset_top;\n"
-"uniform float offset_right;\n"
-"uniform float offset_left;\n"
-/*
- * Used to pass the fade area to the shader
- *
- * [0][0] = x1
- * [0][1] = y1
- * [1][0] = x2
- * [1][1] = y2
- *
- */
-"uniform mat2 fade_area;\n"
-"\n"
-"void main ()\n"
-"{\n"
-" vec4 color = cogl_color_in * texture2D (tex, vec2 (cogl_tex_coord_in[0].xy));\n"
-" float y = height * cogl_tex_coord_in[0].y;\n"
-" float x = width * cogl_tex_coord_in[0].x;\n"
-" float ratio = 1.0;\n"
-" float fade_bottom_start = fade_area[1][1] - offset_bottom;\n"
-" float fade_right_start = fade_area[1][0] - offset_right;\n"
-" float ratio_top = y / offset_top;\n"
-" float ratio_bottom = (fade_area[1][1] - y)/(fade_area[1][1] - fade_bottom_start);\n"
-" float ratio_left = x / offset_left;\n"
-" float ratio_right = (fade_area[1][0] - x)/(fade_area[1][0] - fade_right_start);\n"
-" bool in_scroll_area = fade_area[0][0] <= x && fade_area[1][0] >= x;\n"
-" bool fade_top = y < offset_top && in_scroll_area && (y >= fade_area[0][1]);\n"
-" bool fade_bottom = y > fade_bottom_start && in_scroll_area && (y <= fade_area[1][1]);\n"
-" bool fade_left = x < offset_left && in_scroll_area && (x >= fade_area[0][0]);\n"
-" bool fade_right = x > fade_right_start && in_scroll_area && (x <= fade_area[1][0]);\n"
-"\n"
-" if (fade_top) {\n"
-"  ratio *= ratio_top;\n"
-" }\n"
-"\n"
-" if (fade_bottom) {\n"
-"  ratio *= ratio_bottom;\n"
-" }\n"
-"\n"
-" if (fade_left) {\n"
-"  ratio *= ratio_left;\n"
-" }\n"
-"\n"
-" if (fade_right) {\n"
-"  ratio *= ratio_right;\n"
-" }\n"
-"\n"
-"  cogl_color_out = color * ratio;\n"
-"}";
+#include "st-scroll-view-fade-generated.c"
 
 struct _StScrollViewFade
 {
@@ -107,10 +53,10 @@ struct _StScrollViewFade
   gint height_uniform;
   gint width_uniform;
   gint fade_area_uniform;
-  gint offset_top_uniform;
-  gint offset_bottom_uniform;
-  gint offset_left_uniform;
-  gint offset_right_uniform;
+  gint vfade_offset_uniform;
+  gint hfade_offset_uniform;
+  gint vvalue_uniform;
+  gint hvalue_uniform;
 
   StAdjustment *vadjustment;
   StAdjustment *hadjustment;
@@ -175,14 +121,14 @@ st_scroll_view_fade_pre_paint (ClutterEffect *effect)
         cogl_program_get_uniform_location (self->program, "width");
       self->fade_area_uniform =
         cogl_program_get_uniform_location (self->program, "fade_area");
-      self->offset_top_uniform =
-        cogl_program_get_uniform_location (self->program, "offset_top");
-      self->offset_bottom_uniform =
-        cogl_program_get_uniform_location (self->program, "offset_bottom");
-      self->offset_left_uniform =
-        cogl_program_get_uniform_location (self->program, "offset_left");
-      self->offset_right_uniform =
-        cogl_program_get_uniform_location (self->program, "offset_right");
+      self->vfade_offset_uniform =
+        cogl_program_get_uniform_location (self->program, "vfade_offset");
+      self->hfade_offset_uniform =
+        cogl_program_get_uniform_location (self->program, "hfade_offset");
+      self->vvalue_uniform =
+        cogl_program_get_uniform_location (self->program, "vvalue");
+      self->hvalue_uniform =
+        cogl_program_get_uniform_location (self->program, "hvalue");
     }
 
   parent_class = CLUTTER_EFFECT_CLASS (st_scroll_view_fade_parent_class);
@@ -261,38 +207,24 @@ st_scroll_view_fade_paint_target (ClutterOffscreenEffect *effect)
   if (h_scroll_visible)
       fade_area[1][1] -= clutter_actor_get_height (hscroll);
 
-  st_adjustment_get_values (self->vadjustment, &value, &lower, &upper, NULL, NULL, &page_size);
+  if (self->vvalue_uniform > -1)
+    {
+      st_adjustment_get_values (self->vadjustment, &value, &lower, &upper, NULL, NULL, &page_size);
+      value = (value - lower) / (upper - page_size - lower);
+      cogl_program_set_uniform_1f (self->program, self->vvalue_uniform, value);
+    }
 
-  if (self->offset_top_uniform > -1) {
-    if (value > lower + 0.1)
-      cogl_program_set_uniform_1f (self->program, self->offset_top_uniform, self->vfade_offset);
-    else
-      cogl_program_set_uniform_1f (self->program, self->offset_top_uniform, 0.0f);
-  }
-
-  if (self->offset_bottom_uniform > -1) {
-    if (value < upper - page_size - 0.1)
-      cogl_program_set_uniform_1f (self->program, self->offset_bottom_uniform, self->vfade_offset);
-    else
-      cogl_program_set_uniform_1f (self->program, self->offset_bottom_uniform, 0.0f);
-  }
-
-  st_adjustment_get_values (self->hadjustment, &value, &lower, &upper, NULL, NULL, &page_size);
-
-  if (self->offset_left_uniform > -1) {
-    if (value > lower + 0.1)
-      cogl_program_set_uniform_1f (self->program, self->offset_left_uniform, self->hfade_offset);
-    else
-      cogl_program_set_uniform_1f (self->program, self->offset_left_uniform, 0.0f);
-  }
-
-  if (self->offset_right_uniform > -1) {
-    if (value < upper - page_size - 0.1)
-      cogl_program_set_uniform_1f (self->program, self->offset_right_uniform, self->hfade_offset);
-    else
-      cogl_program_set_uniform_1f (self->program, self->offset_right_uniform, 0.0f);
-  }
+  if (self->hvalue_uniform > -1)
+    {
+      st_adjustment_get_values (self->hadjustment, &value, &lower, &upper, NULL, NULL, &page_size);
+      value = (value - lower) / (upper - page_size - lower);
+      cogl_program_set_uniform_1f (self->program, self->hvalue_uniform, value);
+    }
 
+  if (self->vfade_offset_uniform > -1)
+    cogl_program_set_uniform_1f (self->program, self->vfade_offset_uniform, self->vfade_offset);
+  if (self->hfade_offset_uniform > -1)
+    cogl_program_set_uniform_1f (self->program, self->hfade_offset_uniform, self->hfade_offset);
   if (self->tex_uniform > -1)
     cogl_program_set_uniform_1i (self->program, self->tex_uniform, 0);
   if (self->height_uniform > -1)
@@ -549,7 +481,7 @@ st_scroll_view_fade_init (StScrollViewFade *self)
       if (clutter_feature_available (CLUTTER_FEATURE_SHADERS_GLSL))
         {
           shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
-          cogl_shader_source (shader, fade_glsl_shader);
+          cogl_shader_source (shader, (const char *) st_scroll_view_fade_glsl);
           cogl_shader_compile (shader);
           if (!cogl_shader_is_compiled (shader))
             {
@@ -571,8 +503,10 @@ st_scroll_view_fade_init (StScrollViewFade *self)
   self->height_uniform = -1;
   self->width_uniform = -1;
   self->fade_area_uniform = -1;
-  self->offset_top_uniform = -1;
-  self->offset_bottom_uniform = -1;
+  self->vfade_offset_uniform = -1;
+  self->hfade_offset_uniform = -1;
+  self->vvalue_uniform = -1;
+  self->hvalue_uniform = -1;
   self->vfade_offset = DEFAULT_FADE_OFFSET;
   self->hfade_offset = DEFAULT_FADE_OFFSET;
 
